﻿## Demo_SortFilesByVersionFolderNames.ps1
## This script demonstrates a method of identifying the most recent version of a resource available
##     among a collection of folders each named with their version number.

#region Set up the demo by downloading all versions of the AzureRm.Profile module to the local computer

# Define the location where modules are/will be stored
[string]$psModulesRootFolder = "C:\Temp\Demo-PSModules"

# Display all versions of the AzureRm.Profile module available on the repository
# Note - at the time of writing, the highest available version on the PSGallery is v2.2.0
Find-Module -Name AzureRm.Profile -AllVersions -Repository PSGallery

# Download all versions of the AzureRm.Profile module to the local module folder
# Note - the foreach-object loop is required, otherwise only the most recent version will be downloaded
Find-Module -Name AzureRm.Profile -AllVersions -Repository PSGallery |
    %{Save-Module -InputObject $_ -Path $psModulesRootFolder -Force}

# Load the list of modules available in the folder
$psModuleFolders = Get-ChildItem -Path $psModulesRootFolder -Directory

# Isolate the first module for the demo
$psModuleFolder = $psModuleFolders | Select-Object -First 1

#endregion

#region Illustrate the Sorting Problem

# Find all of the .psd1 files in the module folder
$moduleFiles = Get-ChildItem -Path $psModuleFolder.FullName -Filter "*.psd1" -Recurse

# Sort by file path with the "newest" at the top
$moduleFiles |
    Sort-Object -Property FullName -Descending |
    Select-Object -ExpandProperty FullName

# Notice that in this sorting, v1.0.10 is seen as a LOWER version than v1.0.2.

# To better illustrate this, let's make a version 2.10.0, which SHOULD be seen as the highest version
$demoFolder = $psModuleFolder |
    Get-ChildItem -Directory |
    ?{$_.Name -eq '2.2.0'}
$demoFolder |
    Copy-Item -Destination ((Split-Path -Path $demoFolder.FullName -Parent) + '\2.10.0') -Recurse

# Find all the .psd1 files again and sort by file path with the "newest" at the top
$moduleFiles = Get-ChildItem -Path $psModuleFolder.FullName -Filter "*.psd1" -Recurse
$moduleFiles |
    Sort-Object -Property FullName -Descending |
    Select-Object -ExpandProperty FullName

# v2.2.0 is at the top of the list, while it should be v2.10.0

# Select the "most recent version", sorting by file path
$moduleFile = $moduleFiles |
    Sort-Object -Property FullName -Descending |
    Select-Object -First 1

# Write the file name to the host to show which was selected
$moduleFile.FullName

# v2.2.0 is selected despite not being the most recent version

#endregion

#region Find the Most Recent Version of the Module

# Identify the most recent version available
$mostRecent = ($moduleFiles |
    Select-Object -ExpandProperty FullName |
    Split-Path -Parent |
    Split-Path -Leaf |
    %{[version]$_} |
    Sort-Object -Descending |
    Select-Object -First 1).ToString()

# Grab the most recent version file
$moduleFile = $moduleFiles |
    ?{
        ($_ |
        Select-Object -ExpandProperty FullName |
        Split-Path -Parent |
        Split-Path -Leaf) -eq $mostRecent
    }

# Write the file name to the host to show which was selected
$moduleFile.FullName

#endregion

#region Import the Module

# Import the most recent version of the module
Import-Module -Name $moduleFile.FullName -ErrorAction Stop

#endregion
